//This macro set can be used for calculation of system gain, 
//dark current, and read noise of the detector

var ImageList = newArray(nImages);
var Ggain = 1.0;

//Checks for opened windows 
function PopulateImageList(){ 
	if (nImages==0)
		exit("No images opened");
  	else {
		if (ImageList.length != nImages)
			ImageList = newArray(nImages);	
		for (i=1; i<=nImages; i++) {
        		selectImage(i);
        		ImageList[i-1]=getTitle();
     		}
  	}
}
//This macro calculates detector system gain
//based on the dependence between mean value of image 
// intensity and its variance. 
//
macro "System gain" {
//Check for opened windows 	
	PopulateImageList();
//Reads in bias image and stacks of images
	Dialog.create("Detector calibration");
	Dialog.addChoice("Stack  I",ImageList,"+");
	Dialog.addChoice("Stack II",ImageList,"+");
	Dialog.addChoice("Bias image",ImageList,"+");
	Dialog.show();
	image1 = Dialog.getChoice();
	image2 = Dialog.getChoice();
	bias = Dialog.getChoice();
//Measures mean bias value	
	selectWindow(bias);
 	getStatistics(area, mean, min, max, std, histogram);
	mean_bias=mean;
	selectWindow(image1);
	mean_arr = newArray(nSlices);
	var_arr = newArray(nSlices);
//Calculates sum and difference stacks
	imageCalculator("Subtract create 32-bit stack", image1,image2);
	rename("Difference");
	imageCalculator("Add create 32-bit stack", image1,image2);
	rename("Sum");
//Calculates values for mean and variance for each slice
	for(i=0; i<nSlices; i++) {
		selectWindow("Sum");
		run("Restore Selection");
		setSlice(i+1);
		getStatistics(area, mean, min, max, std, histogram);
		mean_arr[i]=mean-mean_bias;
		selectWindow("Difference");
		run("Restore Selection");
		setSlice(i+1);
		getStatistics(area, mean, min, max, std, histogram);
		var_arr[i]=std*std;
	}
//Closes sum and difference stacks
	selectWindow("Sum"); 
        run("Close"); 
	selectWindow("Difference"); 
        run("Close"); 
//Fits mean againt variance to get gain value
	x=mean_arr;
	y=var_arr;
	Fit.doFit("Straight Line", x, y);
	Fit.plot();
//Prints out gain value		
	Ggain = 1/Fit.p(1);
	print("Gain: ", d2s(Ggain,4), "e-/adu");
	}
//This macro calculates dark current of the camera
//based on the camera system gain, bias image and 
//dark image with known exposure time
//
macro "Dark current" {
//Check for opened windows 
	PopulateImageList();
//Reads in bias and dark images, camera gain, and exposure time	
	Dialog.create("Dark current");
	Dialog.addChoice("Bias image",ImageList,"+");
	Dialog.addChoice("Dark image",ImageList,"+");
	Dialog.addNumber("Exposure, s", 1);
	Dialog.addNumber("Gain, e-/adu", Ggain);
	Dialog.show();
	bias = Dialog.getChoice();
	dark = Dialog.getChoice();
	exposure  = Dialog.getNumber();
	gain  = Dialog.getNumber();
	selectWindow(bias);
//Measures mean bias value	
 	getStatistics(area, mean, min, max, std, histogram);
	mean_bias=mean;
//Measures mean dark value	
	selectWindow(dark);
 	getStatistics(area, mean, min, max, std, histogram);
	mean_dark=mean;
//Calculates and print out dark current	
	dark_current=gain*(mean_dark-mean_bias)/exposure;
	print("Dark current: ", dark_current, "e-/pixel/s");
}

//This macro calculates values of mean and median 
//readout noise of the camera based on the value of 
//the camera system gain and sequence of dark images
//
macro "Read noise" {
//Check for opened windows 
	PopulateImageList();
//Reads in sequence of dark images and value of camera gain		
	Dialog.create("Read noise");
	Dialog.addChoice("Bias sequence",ImageList,"+");
	Dialog.addNumber("Gain, e-/adu", Ggain);
	Dialog.show();	
	bias_seq = Dialog.getChoice();
	gain  = Dialog.getNumber();
//Calculates standard deviation of image intensity in every pixel		
	selectWindow(bias_seq);
	nmax=nSlices;
	run("Z Project...", "start=1 stop=" + nmax + " projection=[Standard Deviation]");
//Converts grey values to electrons	
	run("Multiply...", "value="+gain);
//Makes statistics of the image	
	getStatistics(area, mean, min, max, std, histogram);
       	if ((bitDepth==8) || (bitDepth==24))
		Plot.create("Histogram", "Value", "Count", histogram);
       	else {
		values = newArray(256);
		value = min;
		binWidth = (max-min)/256;
		for (i=0; i<256; i++) {
			values[i] = value;
			value += binWidth;
		}
       	}
//Clear Results window	
	if (isOpen("Results")) { 
		selectWindow("Results"); 
		run("Clear Results");; 
    	} 
//Plots read noise histogram	          
        Plot.create("Histogram", "Read noise, e-", "Counts", values, histogram);
 	run("Set Measurements...", "  mean median redirect=None decimal=3");
	run("Measure");
	close();
//Extracts mean and median read noise values	
	mean = getResult("Mean",0);
	median = getResult("Median",0);
//Close Results window	
	if (isOpen("Results")) { 
		selectWindow("Results"); 
		run("Close"); 
    	} 
//Prints out mean and median values of read noise		
	print("Read noise: "); 
	print("   Mean:    ", mean, "e-"); 
	print("   Median: ", median, "e-"); 
}